@@ -24,10 +24,6 @@
     See the License for the specific language governing permissions and
     limitations under the License.
 """
-from __future__ import division
-
-from builtins import str
-
 import numpy as np
 import logging
 
@@ -40,7 +36,7 @@ from mslib.utils import config_loader, rotate_point
 
 class HexagonException(Exception):
     def __init__(self, error_string):
-        logging.debug("{}".format(error_string))
+        logging.debug("%s", error_string)
 
 
 def create_hexagon(center_lat, center_lon, radius, angle=0.):
@@ -107,7 +103,7 @@ class HexagonControlWidget(QtWidgets.QWidget, ui.Ui_HexagonDockWidget):
         waypoints = []
         for i, point in enumerate(points):
             waypoints.append(
-                ft.Waypoint(lon=float(round(point[1], 2)), lat=float(round(point[0], 2)),
+                ft.Waypoint(lon=float(point[1]), lat=float(point[0]),
                             flightlevel=float(flightlevel), comments="Hexagon {:d}".format(i + 1)))
         waypoints_model.insertRows(row, rows=len(waypoints), waypoints=waypoints)
         index = waypoints_model.index(row, 0)
