@@ -32,17 +32,12 @@
     limitations under the License.
 """
 
-from __future__ import division
-
-from builtins import str
-
 import logging
 import numpy as np
 import matplotlib
 import matplotlib.path as mpath
 import matplotlib.patches as mpatches
 import matplotlib.pyplot as plt
-from matplotlib import patheffects
 import mpl_toolkits.basemap as basemap
 try:
     import mpl_toolkits.basemap.pyproj as pyproj
@@ -143,7 +138,7 @@ class MapCanvas(basemap.Basemap):
             try:
                 self._draw_auto_graticule()
             except Exception as ex:
-                logging.error(u"ERROR: cannot plot graticule (message: {} - '{}')".format(type(ex), ex))
+                logging.error("ERROR: cannot plot graticule (message: %s - '%s')", type(ex), ex)
         else:
             self.map_parallels = None
             self.map_meridians = None
@@ -461,7 +456,7 @@ class MapCanvas(basemap.Basemap):
             try:
                 self.map_boundary.remove()
             except NotImplementedError as ex:
-                logging.debug("{}".format(ex))
+                logging.debug("%s", ex)
             self.map_boundary = None
 
         cont_vis = self.appearance["fill_continents"]
@@ -681,7 +676,7 @@ class MapCanvas(basemap.Basemap):
                     elif element == "lineplot":
                         plt.setp(
                             item.getGxElementProperty(element,
-                                                      u"instance::{}".format(self.identifier)),
+                                                      "instance::{}".format(self.identifier)),
                             visible=item_properties["visible"],
                             color=item_properties["colour"],
                             linestyle=item_properties["linestyle"],
@@ -689,7 +684,7 @@ class MapCanvas(basemap.Basemap):
                     elif element == "timemarker":
                         plt.setp(
                             item.getGxElementProperty(element,
-                                                      u"instance::{}".format(self.identifier)),
+                                                      "instance::{}".format(self.identifier)),
                             visible=item_properties["visible"],
                             color=item_properties["colour"])
 
@@ -707,7 +702,7 @@ class MapCanvas(basemap.Basemap):
                     # Remove old plot instances.
                     try:
                         instances = item.getGxElementProperty("lineplot",
-                                                              u"instance::{}".format(self.identifier))
+                                                              "instance::{}".format(self.identifier))
                         for instance in instances:
                             instance.remove()
                     except (KeyError, ValueError) as error:
@@ -719,7 +714,7 @@ class MapCanvas(basemap.Basemap):
                                               linestyle=item_properties["linestyle"],
                                               linewidth=item_properties["linewidth"])
                     item.setGxElementProperty("lineplot",
-                                              u"instance::{}".format(self.identifier),
+                                              "instance::{}".format(self.identifier),
                                               plot_instance)
 
                 # Get the indexes for the time markers. If no time markers should
@@ -728,7 +723,7 @@ class MapCanvas(basemap.Basemap):
                     # Try to remove an existing time marker instance from
                     # the plot.
                     old_scatter_instance = item.getGxElementProperty("timemarker",
-                                                                     u"instance::{}".format(self.identifier))
+                                                                     "instance::{}".format(self.identifier))
                     plt.setp(old_scatter_instance, visible=False)
                     old_scatter_instance.remove()
                 except (KeyError, ValueError) as error:
@@ -739,7 +734,7 @@ class MapCanvas(basemap.Basemap):
                                                     s=20, c=item_properties["colour"],
                                                     visible=item_properties["visible"], linewidth=0)
                     item.setGxElementProperty("timemarker",
-                                              u"instance::{}".format(self.identifier),
+                                              "instance::{}".format(self.identifier),
                                               scatter_instance)
 
         # Update the figure canvas.
@@ -922,145 +917,3 @@ class SatelliteOverpassPatch(object):
                 element.remove()
             except Exception as ex:
                 logging.error("Wildcard exception caught: %s %s", type(ex), ex)
-
-
-class KMLPatch(object):
-    """
-    Represents a KML overlay.
-
-    KML overlay implementation is currently very crude and basic and most features are not supported.
-    """
-
-    def __init__(self, mapcanvas, kml, overwrite=False, color="red", linewidth=1):
-        self.map = mapcanvas
-        self.kml = kml
-        self.patches = []
-        self.color = color
-        self.linewidth = linewidth
-        self.overwrite = overwrite
-        self.draw()
-
-    def compute_xy(self, coordinates):
-        coords = str(coordinates).split()
-        lons, lats = [[float(_x.split(",")[_i]) for _x in coords] for _i in range(2)]
-        return self.map(lons, lats)
-
-    def add_polygon(self, polygon, style, _):
-        """
-        Plot KML polygons
-
-        :param polygon: pykml object specifying a polygon
-        """
-        kwargs = self.styles.get(style, {}).get("PolyStyle", {"linewidth": self.linewidth, "color": self.color})
-        for boundary in ["outerBoundaryIs", "innerBoundaryIs"]:
-            if hasattr(polygon, boundary):
-                x, y = self.compute_xy(getattr(polygon, boundary).LinearRing.coordinates)
-                self.patches.append(self.map.plot(x, y, "-", zorder=10, **kwargs))
-
-    def add_point(self, point, style, name):
-        """
-        Plot KML point
-
-        :param point: pykml object specifying point
-        :param name: name of placemark for annotation
-        """
-        x, y = self.compute_xy(point.coordinates)
-        self.patches.append(self.map.plot(x[0], y[0], "o", zorder=10, color=self.color))
-        if name is not None:
-            self.patches.append([self.map.ax.annotate(
-                name, xy=(x[0], y[0]), xycoords="data", xytext=(5, 5), textcoords='offset points', zorder=10,
-                path_effects=[patheffects.withStroke(linewidth=2, foreground='w')])])
-
-    def add_line(self, line, style, _):
-        """
-        Plot KML line
-
-        :param line: pykml LineString object
-        """
-        kwargs = self.styles.get(style, {}).get("LineStyle", {"linewidth": self.linewidth, "color": self.color})
-        x, y = self.compute_xy(line.coordinates)
-        self.patches.append(self.map.plot(x, y, "-", zorder=10, **kwargs))
-
-    def parse_geometries(self, placemark, style=None, name=None):
-        if style.startswith("#"):
-            # Remove # at beginning of style marking a locally defined style.
-            # general urls for styles are not supported
-            style = style[1:]
-        for attr_name, method in (
-                ("Point", self.add_point),
-                ("Polygon", self.add_polygon),
-                ("LineString", self.add_line),
-                ("MultiGeometry", self.parse_geometries)):
-            for attr in getattr(placemark, attr_name, []):
-                logging.debug("Found %s", attr_name)
-                method(attr, style, name)
-
-    def parse_placemarks(self, level):
-        for placemark in getattr(level, "Placemark", []):
-            name = getattr(placemark, "name", None)
-            style = str(getattr(placemark, "styleUrl", ""))
-            logging.debug("Placemark: %s %s", style, name)
-            self.parse_geometries(placemark, style, name)
-        for folder in getattr(level, "Folder", []):
-            name = getattr(folder, "name", None)
-            logging.debug("Folder: %s", name)
-            self.parse_placemarks(folder)
-
-    def get_style_params(self, style):
-        result = {
-            "color": str(getattr(style, "color", "")),
-            "linewidth": float(getattr(style, "width", self.linewidth))
-        }
-        if not result["color"] or len(result["color"]) != 8:
-            result["color"] = self.color
-        else:
-            result["color"] = [(int(result["color"][i:i + 2], 16) / 255.) for i in range(0, 8, 2)]
-        return result
-
-    def parse_styles(self, level):
-        for style in getattr(level, "Style", []):
-            name = style.attrib.get("id")
-            if name is None:
-                continue
-            self.styles[name] = {
-                "LineStyle": self.get_style_params(getattr(style, "LineStyle", None)),
-                "PolyStyle": self.get_style_params(getattr(style, "PolyStyle", None))
-            }
-        for folder in getattr(level, "Folder", []):
-            name = getattr(folder, "name", None)
-            logging.debug("Folder: %s", name)
-            self.parse_styles(folder)
-
-    def draw(self):
-        """Do the actual plotting of the patch.
-        """
-        # Plot satellite track.
-        self.styles = {}
-        if not self.overwrite:
-            self.parse_styles(self.kml.Document)
-        self.parse_placemarks(self.kml.Document)
-
-        self.map.ax.figure.canvas.draw()
-
-    def update(self, overwrite=None, color=None, linewidth=None):
-        """Removes the current plot of the patch and redraws the patch.
-           This is necessary, for instance, when the map projection and/or
-           extent has been changed.
-        """
-        if overwrite is not None:
-            self.overwrite = overwrite
-        if color is not None:
-            self.color = color
-        if linewidth is not None:
-            self.linewidth = linewidth
-        self.remove()
-        self.draw()
-
-    def remove(self):
-        """Remove this satellite patch from the map canvas.
-        """
-        for patch in self.patches:
-            for element in patch:
-                element.remove()
-        self.patches = []
-        self.map.ax.figure.canvas.draw()
