@@ -26,8 +26,6 @@
     limitations under the License.
 """
 
-from __future__ import division
-
 # Parts of the code have been adapted from Chapter 6 of Sandro Tosi,
 # 'Matplotlib for Python Developers'.
 
@@ -95,12 +93,12 @@ class MplCanvas(FigureCanvasQTAgg):
         """
         self.default_filename = ""
         if title:
-            self.default_filename += u"_{:>5}".format(title.split()[0])
+            self.default_filename += "_{:>5}".format(title.split()[0])
         if style:
-            title += u' ({})'.format(style)
+            title += " ({})".format(style)
         if level:
-            title += u' at {}'.format(level)
-            self.default_filename += u"_{}".format(level.split()[0])
+            title += " at {}".format(level)
+            self.default_filename += "_{}".format(level.split()[0])
         if isinstance(valid_time, datetime) and isinstance(init_time, datetime):
             time_step = valid_time - init_time
         else:
@@ -114,12 +112,12 @@ class MplCanvas(FigureCanvasQTAgg):
         if valid_time:
             if init_time:
                 if time_step is not None:
-                    title += u"\nValid: {} (step {:d} hrs from {})".format(
+                    title += "\nValid: {} (step {:d} hrs from {})".format(
                         valid_time, (time_step.days * 86400 + time_step.seconds) // 3600, init_time)
                 else:
-                    title += u"\nValid: {} (initialisation: {})".format(valid_time, init_time)
+                    title += "\nValid: {} (initialisation: {})".format(valid_time, init_time)
             else:
-                title += u"\nValid: {}".format(valid_time)
+                title += "\nValid: {}".format(valid_time)
 
         # Set title.
         self.ax.set_title(title, horizontalalignment='left', x=0)
@@ -159,8 +157,8 @@ class MplWidget(QtWidgets.QWidget):
         self.setLayout(self.vbl)
 
 
-def _getSaveFileName(parent, title="Choose a filename to save to", filename=u"test.png",
-                     filters=u" Images (*.png)"):
+def _getSaveFileName(parent, title="Choose a filename to save to", filename="test.png",
+                     filters=" Images (*.png)"):
     _dirname, _name = os.path.split(filename)
     _dirname = os.path.join(_dirname, "")
     return getSaveFileNameAndFilter(parent, fs_url=_dirname, file_pattern=filters,
@@ -220,12 +218,24 @@ NavigationToolbar2QT.save_figure = save_figure
 
 
 class NavigationToolbar(NavigationToolbar2QT):
-    # only display the buttons we need
+    """
+    parts of this class have been copied from the NavigationToolbar2QT class.
+
+    According to https://matplotlib.org/users/license.html we shall
+    summarise our changes to matplotlib code:
+
+    We copied small parts of the given implementation of the navigation
+    toolbar class to allow for our custom waypoint buttons. Our code extends
+    the matplotlib toolbar to allow for less or additional buttons and properly
+    update all plots and elements in case the pan or zoom elements were
+    triggered by the user.
+    """
 
     def __init__(self, canvas, parent, sideview=False, coordinates=True):
         if sideview:
             self.toolitems = [
                 _x for _x in NavigationToolbar2QT.toolitems if _x[0] in ('Save',)]
+            self.set_history_buttons = lambda: None
         else:
             self.toolitems = [
                 _x for _x in NavigationToolbar2QT.toolitems if
@@ -377,7 +387,7 @@ class NavigationToolbar(NavigationToolbar2QT):
                 try:
                     lat, lon = self.canvas.waypoints_interactor.get_lat_lon(event)
                 except (ValueError, OverflowError) as ex:
-                    logging.error("{}".format(ex))
+                    logging.error("%s", ex)
                 else:
                     s = "lat={:6.2f}, lon={:7.2f}".format(lat, lon)
                     artists = [a for a in event.inaxes._mouseover_set
@@ -434,8 +444,6 @@ class NavigationToolbar(NavigationToolbar2QT):
             a.setCheckable(True)
             a.setToolTip(tooltip_text)
 
-        self.buttons = {}
-
         # Add the x,y location widget at the right side of the toolbar
         # The stretch factor is 1 which means any resizing of the toolbar
         # will resize this label instead of the buttons.
@@ -449,9 +457,6 @@ class NavigationToolbar(NavigationToolbar2QT):
             labelAction = self.addWidget(self.locLabel)
             labelAction.setVisible(True)
 
-        # reference holder for subplots_adjust window
-        self.adj_window = None
-
         # Esthetic adjustments - we need to set these explicitly in PyQt5
         # otherwise the layout looks different - but we don't want to set it if
         # not using HiDPI icons otherwise they look worse than before.
@@ -503,6 +508,8 @@ class MplSideViewCanvas(MplCanvas):
     """Specialised MplCanvas that draws a side view (vertical section) of a
        flight track / list of waypoints.
     """
+    _pres_maj = np.concatenate([np.arange(top * 10, top, -top) for top in (10000, 1000, 100, 10)] + [[10]])
+    _pres_min = np.concatenate([np.arange(top * 10, top, -top // 10) for top in (10000, 1000, 100, 10)] + [[10]])
 
     _pres_maj = np.concatenate([np.arange(top * 10, top, -top) for top in (10000, 1000, 100, 10)] + [[10]])
     _pres_min = np.concatenate([np.arange(top * 10, top, -top // 10) for top in (10000, 1000, 100, 10)] + [[10]])
@@ -524,9 +531,11 @@ class MplSideViewCanvas(MplCanvas):
                               "draw_flighttrack": True,
                               "fill_flighttrack": True,
                               "label_flighttrack": True,
+                              "draw_ceiling": True,
                               "colour_ft_vertices": (0, 0, 1, 1),
                               "colour_ft_waypoints": (1, 0, 0, 1),
-                              "colour_ft_fill": (0, 0, 1, 0.15)}
+                              "colour_ft_fill": (0, 0, 1, 0.15),
+                              "colour_ceiling": (0, 0, 1, 0.15)}
         if settings is not None:
             self.settings_dict.update(settings)
 
@@ -541,8 +550,10 @@ class MplSideViewCanvas(MplCanvas):
         self.draw_flight_levels()
         self.imgax = None
         self.image = None
+        self.ceiling_alt = []
         # If a waypoints model has been passed, create an interactor on it.
         self.waypoints_interactor = None
+        self.waypoints_model = None
         self.basename = "sideview"
         if model:
             self.set_waypoints_model(model)
@@ -584,17 +595,29 @@ class MplSideViewCanvas(MplCanvas):
                 labels = ["" if x.split(".")[-1][0] in "9" else x for x in labels]
             self.ax.set_ylabel("pressure (hPa)")
         elif vaxis == "pressure altitude":
-            major_heights = np.arange(0, thermolib.pressure2flightlevel(self.p_top) * 0.03048, 2)
-            minor_heights = np.arange(0, thermolib.pressure2flightlevel(self.p_top) * 0.03048, 0.5)
-            major_fl = 10 * major_heights / 0.3048
-            minor_fl = 10 * minor_heights / 0.3048
-            major_ticks = thermolib.flightlevel2pressure_a(major_fl)
-            minor_ticks = thermolib.flightlevel2pressure_a(minor_fl)
+            bot_km = thermolib.pressure2flightlevel(self.p_bot) * 0.03048
+            top_km = thermolib.pressure2flightlevel(self.p_top) * 0.03048
+            ma_dist, mi_dist = 4, 1.0
+            if (top_km - bot_km) <= 20:
+                ma_dist, mi_dist = 1, 0.5
+            elif (top_km - bot_km) <= 40:
+                ma_dist, mi_dist = 2, 0.5
+            major_heights = np.arange(0, top_km + 1, ma_dist)
+            minor_heights = np.arange(0, top_km + 1, mi_dist)
+            major_ticks = thermolib.flightlevel2pressure_a(major_heights / 0.03048)
+            minor_ticks = thermolib.flightlevel2pressure_a(minor_heights / 0.03048)
             labels = major_heights
             self.ax.set_ylabel("pressure altitude (km)")
         elif vaxis == "flight level":
-            major_fl = np.arange(0, 2132, 50)
-            minor_fl = np.arange(0, 2132, 10)
+            bot_km = thermolib.pressure2flightlevel(self.p_bot) * 0.03048
+            top_km = thermolib.pressure2flightlevel(self.p_top) * 0.03048
+            ma_dist, mi_dist = 50, 10
+            if (top_km - bot_km) <= 10:
+                ma_dist, mi_dist = 20, 10
+            elif (top_km - bot_km) <= 40:
+                ma_dist, mi_dist = 40, 10
+            major_fl = np.arange(0, 2132, ma_dist)
+            minor_fl = np.arange(0, 2132, mi_dist)
             major_ticks = thermolib.flightlevel2pressure_a(major_fl)
             minor_ticks = thermolib.flightlevel2pressure_a(minor_fl)
             labels = major_fl
@@ -664,6 +687,31 @@ class MplSideViewCanvas(MplCanvas):
                                                   lons[::tick_index_step],
                                                   times[::tick_index_step])],
                                     rotation=25, fontsize=10, horizontalalignment="right")
+
+        for _line in self.ceiling_alt:
+            _line.remove()
+        self.ceiling_alt = []
+        if self.waypoints_model is not None and self.waypoints_interactor is not None:
+            vertices = self.waypoints_interactor.pathpatch.get_path().vertices
+            vx, vy = list(zip(*vertices))
+            wpd = self.waypoints_model.all_waypoint_data()
+            xs, ys = [], []
+            aircraft = self.waypoints_model.performance_settings["aircraft"]
+            for i in range(len(wpd) - 1):
+                weight = np.linspace(wpd[i].weight, wpd[i + 1].weight, 5, endpoint=False)
+                ceil = [aircraft.get_ceiling_altitude(_w) for _w in weight]
+                xs.extend(np.linspace(vx[i], vx[i + 1], 5, endpoint=False))
+                ys.extend(ceil)
+            xs.append(vx[-1])
+            ys.append(aircraft.get_ceiling_altitude(wpd[-1].weight))
+
+            self.ceiling_alt = self.ax.plot(
+                xs, thermolib.flightlevel2pressure_a(np.asarray(ys)),
+                color="k", ls="--")
+            self.update_ceiling(
+                self.settings_dict["draw_ceiling"] and self.waypoints_model.performance_settings["visible"],
+                self.settings_dict["colour_ceiling"])
+
         self.draw()
 
     def set_vertical_extent(self, pbot, ptop):
@@ -706,7 +754,7 @@ class MplSideViewCanvas(MplCanvas):
         for level in self.flightlevels:
             pressure = thermolib.flightlevel2pressure(level)
             self.fl_label_list.append(ax.axhline(pressure, color='k'))
-            self.fl_label_list.append(ax.text(0.1, pressure, u"FL{:d}".format(level)))
+            self.fl_label_list.append(ax.text(0.1, pressure, "FL{:d}".format(level)))
         self.draw()
 
     def get_flight_levels(self):
@@ -727,6 +775,14 @@ class MplSideViewCanvas(MplCanvas):
             gxelement.set_visible(visible)
         self.draw()
 
+    def update_ceiling(self, visible, color):
+        """Toggle the visibility of the flight level lines.
+        """
+        for line in self.ceiling_alt:
+            line.set_color(color)
+            line.set_visible(visible)
+        self.draw()
+
     def get_settings(self):
         """Returns a dictionary containing settings regarding the side view
            appearance.
@@ -742,6 +798,12 @@ class MplSideViewCanvas(MplCanvas):
         self.set_flight_levels(settings["flightlevels"])
         self.set_vertical_extent(*settings["vertical_extent"])
         self.set_flight_levels_visible(settings["draw_flightlevels"])
+        self.update_ceiling(
+            settings["draw_ceiling"] and (
+                self.waypoints_model is not None and
+                self.waypoints_model.performance_settings["visible"]),
+            settings["colour_ceiling"])
+
         if self.waypoints_interactor is not None:
             self.waypoints_interactor.set_vertices_visible(
                 settings["draw_flighttrack"])
@@ -792,7 +854,7 @@ class MplSideViewCanvas(MplCanvas):
         """
         logging.debug("plotting vertical section image..")
         ix, iy = img.size
-        logging.debug(u"  image size is {:d}{:d} px, format is {}".format(ix, iy, img.format))
+        logging.debug("  image size is %dx%d px, format is '%s'", ix, iy, img.format)
         # Test if the image axes exist. If not, create them.
         if self.imgax is None:
             # Disable old white figure background so that the new underlying
@@ -812,8 +874,8 @@ class MplSideViewCanvas(MplCanvas):
             self.image.remove()
 
         # Plot the new image in the image axes and adjust the axes limits.
-        self.image = self.imgax.imshow(img, interpolation="nearest", aspect="auto",
-                                       origin=PIL_IMAGE_ORIGIN)
+        self.image = self.imgax.imshow(
+            img, interpolation="nearest", aspect="auto", origin=PIL_IMAGE_ORIGIN)
         self.imgax.set_xlim(0, ix - 1)
         self.imgax.set_ylim(iy - 1, 0)
         self.draw()
@@ -861,7 +923,7 @@ class MplTopViewCanvas(MplCanvas):
         """
         super(MplTopViewCanvas, self).__init__()
         self.waypoints_interactor = None
-        self.satoverpasspatch = None
+        self.satoverpasspatch = []
         self.kmloverlay = None
         self.map = None
         self.basename = "topview"
@@ -952,8 +1014,8 @@ class MplTopViewCanvas(MplCanvas):
         self.pdlg.setValue(8)
         QtWidgets.QApplication.processEvents()
 
-        if self.satoverpasspatch:
-            self.satoverpasspatch.update()
+        for segment in self.satoverpasspatch:
+            segment.update()
 
         if self.kmloverlay:
             self.kmloverlay.update()
@@ -1013,8 +1075,7 @@ class MplTopViewCanvas(MplCanvas):
         """Draw the image img on the current plot.
         """
         logging.debug("plotting image..")
-        self.wms_image = self.map.imshow(img, interpolation="nearest", alpha=1.,
-                                         origin=PIL_IMAGE_ORIGIN)
+        self.wms_image = self.map.imshow(img, interpolation="nearest", origin=PIL_IMAGE_ORIGIN)
         # NOTE: imshow always draws the images to the lowest z-level of the
         # plot.
         # See these mailing list entries:
@@ -1062,24 +1123,25 @@ class MplTopViewCanvas(MplCanvas):
                 self.legax.set_position([1 - ax_extent_x, 0.01, ax_extent_x, ax_extent_y])
 
             # Plot the new legimg in the legax axes.
-            self.legimg = self.legax.imshow(img, origin=PIL_IMAGE_ORIGIN, aspect="equal",
-                                            interpolation="nearest")
+            self.legimg = self.legax.imshow(img, origin=PIL_IMAGE_ORIGIN, aspect="equal", interpolation="nearest")
         self.draw()
         # required so that it is actually drawn...
         QtWidgets.QApplication.processEvents()
 
-    def plot_satellite_overpass(self, segment):
+    def plot_satellite_overpass(self, segments):
         """Plots a satellite track on top of the map.
         """
-        if self.satoverpasspatch:
-            # If track is currently plotted on the map, remove it.
-            self.satoverpasspatch.remove()
-            if not segment:
-                self.satoverpasspatch = None
-                self.draw()
-        if segment:
+        # If track is currently plotted on the map, remove it.
+        for segment in self.satoverpasspatch:
+            segment.remove()
+        self.satoverpasspatch = []
+
+        if segments:
             # Create a new patch.
-            self.satoverpasspatch = mpl_map.SatelliteOverpassPatch(self.map, segment)
+            self.satoverpasspatch = [
+                mpl_map.SatelliteOverpassPatch(self.map, segment)
+                for segment in segments]
+        self.draw()
 
     def plot_kml(self, kmloverlay):
         """Plots a satellite track on top of the map.
@@ -1285,7 +1347,7 @@ class MplTimeSeriesViewCanvas(MplCanvas):
         # Iterative list traversal no. 2: Draw / update plots.
         for item in itemsList:
 
-            logging.debug(u"plotting item {}".format(item.getName()))
+            logging.debug("plotting item %s", item.getName())
             # The variables that have to be plotted are children of self.mapItem.
             # Plot all variables whose visible-flag is set to True.
             variablesToPlot = [variable for variable in item.childItems
@@ -1297,7 +1359,7 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                 for variable in variablesToPlot:
                     plt.setp(
                         variable.getGxElementProperty(
-                            "timeseries", u"instance::{}".format(self.identifier)),
+                            "timeseries", "instance::{}".format(self.identifier)),
                         visible=item.isVisible(self.identifier),
                         color=item.getGxElementProperty("general", "colour"),
                         linestyle=item.getGxElementProperty("general", "linestyle"),
@@ -1327,7 +1389,7 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                         linestyle=item.getGxElementProperty("general", "linestyle"),
                         linewidth=item.getGxElementProperty("general", "linewidth"))
                     variable.setGxElementProperty(
-                        "timeseries", u"instance::{}".format(self.identifier), plotInstance)
+                        "timeseries", "instance::{}".format(self.identifier), plotInstance)
                     #
                     # If we've just plotted the pressure variable flip the y-axis
                     # upside down.
@@ -1353,10 +1415,10 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                     # The topmost subplot gets the title (the name of self.mapItem).
                     if index == 0:
                         if len(itemsList) > 1:
-                            ax.set_title(u"Ensemble: {} to {}".format(
+                            ax.set_title("Ensemble: {} to {}".format(
                                 itemsList[0].getName(), itemsList[-1].getName()))
                         else:
-                            ax.set_title(u"Single item: {}".format(itemsList[0].getName()))
+                            ax.set_title("Single item: {}".format(itemsList[0].getName()))
 
                     #
                     # The bottommost subplot gets the x-label: The name of the time
